/*
  GaussianND.h

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>

  Evaluates multidimensional Gaussians
*/

#ifndef __GAUSSIANND_H_
#define __GAUSSIANND_H_
 
#include <gsl/gsl_linalg.h>
#include <string.h>
 
class GaussianND {
 public:
  /* Argument is dimensionality of Gaussian */
  GaussianND(int dim);
  GaussianND::GaussianND(GaussianND &src);
  ~GaussianND() {
    //	   printf("GaussND: free'ing %p and %p\n", covarCopy, invCovar);
    free(covarCopy);
    free(invCovar);
    gsl_permutation_free(gslPerm);
  }
 
  /* Set mean and covariance */
  void SetMean(double *mean_a) { mean = mean_a; }
  void SetCovar(double *covar_a);
 
  /* Calculate value at vector, store exponent part in exponent_out,
     normalizer part in normalizer_out
   */
  void LnValue(double *vector, double *exponent_out, double *normalizer_out);

  /* Calculate and return log of value at vector */
  double LnValue(double *vector);

  /* Calculate and return value at vector */
  double Value(double *arg);
 
 private:
  int dim;
  double scaler;
  double *mean, *covar;
  double *covarCopy, *invCovar;
  gsl_permutation *gslPerm;
   
};
 
#endif
