/* 
   Quaternion.h

   (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#ifndef __QUATERNION_H__
#define __QUATERNION_H__

#include <stdio.h>

// Note that quaternion real part here is the LAST component 

class Quaternion {
 public:
  Quaternion();
  //  Quaternion(double *quat_ptr);
  Quaternion(double i, double j, double k, double w);
  Quaternion(Quaternion &quat);
  Quaternion(double roll, double pitch, double yaw);

  void MakePositive();
  void Conjugate();
  void Invert();
  void RMultiply(Quaternion *otherquat);
  void Renormalize();

  void IntegrateRotation(double delta_time, double *angular_rates);

  void RotateB2F(double *body_vector, double *fixed_vector_out);
  void RotateF2B(double *fixed_vector, double *body_vector_out);

  double *GetArray() { return quaternion; }
  void RecalcRotationMatrix() { rotationMatrixDirty = true; }
  //  void SetArray(double *quat);

  double *GetMatrixB2F();
  double *GetMatrixF2B();

  void GetEulerAngles(double *angles);

  void Print(FILE *file);

  Quaternion *RDivide(Quaternion *rquat);
  double GetAngleDiff(Quaternion *quat);

 private:
  int rotationCount;
  double *quaternion;
  double quaternion_storage[4];
  double rotationMatrixB2F[3][3];
  double rotationMatrixF2B[3][3];
  bool rotationMatrixDirty;  
};

#endif
