/* 
   RBParticle.h

   (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#include "RBParticle.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include <gsl/gsl_randist.h>
#include <gsl/gsl_cblas.h>

// Very important to make sure each particle's PRNG is seeded differently!
// Otherwise, random noise added to angular velocities will be the same 
// across particles.
static unsigned int prng_seed = 1;

RBParticle::RBParticle(int pos_dim) {
  posDim = pos_dim;

  AllocateMatrices(posDim);

  quaternion = new Quaternion();
  prng = gsl_rng_alloc(gsl_rng_taus2);
  prng_seed += 100 + time(NULL);
  //  prng_seed += 100;
  gsl_rng_set(prng, prng_seed);

  SetWeight(1.0);
  SetLogWeight(0.0);
  memset(rotVeloc, 0, sizeof(double)*3);
}

RBParticle::RBParticle(int pos_dim,
		       double *initial_mean, 
		       double *initial_covar) {
  posDim = pos_dim;

  AllocateMatrices(posDim);

  memcpy(posMean, initial_mean, sizeof(double)*posDim);
  memcpy(posCovar, initial_covar, sizeof(double)*posDim*posDim);

  quaternion = new Quaternion();
  prng_seed += 100 + time(NULL);
  //  prng_seed += 100;
  prng = gsl_rng_alloc(gsl_rng_taus2);
  gsl_rng_set(prng, prng_seed);

  memset(rotVeloc, 0, sizeof(double)*3);
  SetWeight(1.0);
  SetLogWeight(0.0);
}

RBParticle::RBParticle(RBParticle &src) {
  posDim = src.posDim;

  AllocateMatrices(posDim);

  memcpy(posMean, src.posMean, sizeof(double)*posDim);
  memcpy(posCovar, src.posCovar, sizeof(double)*posDim*posDim);

  quaternion = new Quaternion(*(src.GetQuaternion()));
  prng_seed += 100 + time(NULL);
  //  prng_seed += 100;
  prng = gsl_rng_alloc(gsl_rng_taus2);
  gsl_rng_set(prng, prng_seed);

  memcpy(rotVeloc, &(src.rotVeloc), sizeof(double)*3);
  weight = src.weight;
  logWeight = src.logWeight;
}

void RBParticle::Randomize() {
  double angular_rates[3];
  angular_rates[0] = drand48() * 1000;
  angular_rates[1] = drand48() * 1000;
  angular_rates[2] = drand48() * 1000;

  quaternion->IntegrateRotation(100.0, angular_rates);
}

void RBParticle::AllocateMatrices(int pos_dim) {
  posMean = (double*)calloc(pos_dim, sizeof(double));
  posCovar = (double*)calloc(pos_dim*pos_dim, sizeof(double));
}

void RBParticle::SetWeight(double weight_arg) { 
  weight = weight_arg; 
}

void RBParticle::PrintMatrix(double *matrix) {
  fprintf(stderr, "\n[ %f %f %f ]", matrix[0], matrix[1], matrix[2]);
  fprintf(stderr, "\n[ %f %f %f ]", matrix[3], matrix[4], matrix[5]);
  fprintf(stderr, "\n[ %f %f %f ]\n", matrix[6], matrix[7], matrix[8]);
}






