/*
  jittercorrector/JitterCorrector.h

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>

  Builds a queue of messages and deals them out in sequence specified 
  by timestamps.  
*/

#ifndef __JITTER_CORRECTOR_H__
#define __JITTER_CORRECTOR_H__

#include <assert.h>
#include <set>

using namespace std;

#include "JitterMessage.h"

template <class Tpl>
struct stamp_comparer {
  bool operator()(JitterMessage<Tpl> *mess_a, JitterMessage<Tpl> *mess_b) {
    return mess_a->Timestamp() < mess_b->Timestamp();
  }
};

template <class Tpl>
class JitterCorrector {
 public:
  JitterCorrector(double max_message_age) {
    maxMessageAge = max_message_age;
    lastDeliveredMessageTimestamp = 0.0;
  }

  bool EnqueueMessage(double timestamp, Tpl user_data) {
    if (timestamp < lastDeliveredMessageTimestamp) {
      fprintf(stderr, 
	      "WARNING: message is %lf seconds too old--undeliverable\n",
	      lastDeliveredMessageTimestamp - timestamp);
      return false;
    }
      
    JitterMessage<Tpl> *mess = new JitterMessage<Tpl>(timestamp, user_data);

    assert(mess != NULL);

    messages.insert(mess);
    return true;
  }
  
  bool GetMessage(Tpl *out, double *timestamp = NULL) {
    if (messages.empty()) 
      return false;

    /*
    for (typename multiset<JitterMessage<Tpl>*>::iterator mit = messages.begin();
	 mit != messages.end(); mit++) {
      printf(" T - %lf\n", (*mit)->Timestamp());
    }
    */

    JitterMessage<Tpl> *oldest_message = *(messages.begin());
    JitterMessage<Tpl> *newest_message = *(messages.rbegin());

    double oldest_message_age = oldest_message->Age(newest_message);

    assert(oldest_message_age >= 0.0);

    if (oldest_message_age >= maxMessageAge){
      Tpl userdata = oldest_message->UserData();
      messages.erase(messages.begin());
      lastDeliveredMessageTimestamp = oldest_message->Timestamp();

      if (timestamp != NULL) 
	*timestamp = lastDeliveredMessageTimestamp;

      assert(!isnan(lastDeliveredMessageTimestamp));

      delete oldest_message;
      *out = userdata;
      return true;
    } else {
      return false;
    }
  }

 private:
  double maxMessageAge;
  double lastDeliveredMessageTimestamp;
  multiset< JitterMessage<Tpl>*, stamp_comparer<Tpl> > messages;
};

#endif
