/*
  lagr/GPSFixer.h

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>

  Hack to integrate GPS in a timely fashion by projecting them 
  ahead in time.
*/

#ifndef __GPS_FIXER_H__
#define __GPS_FIXER_H__

#include <string.h>
#include <vector>
using namespace std;

class PoseDatum {
 public:
  PoseDatum(double tstamp, double *pos) {
    memcpy(position, pos, sizeof(double)*6);
    timestamp = tstamp;
  }
  
  double position[6]; 
  double timestamp;
};

class GPSFixer {
 public:
  GPSFixer() {
    lastTstamp = 0.0;
  }
  
  void AddEstimate(double tstamp, double *pos) {
    posedata.push_back(new PoseDatum(tstamp, pos));
    lastTstamp = tstamp;
  }

  void SetLatestTimestamp(double tstamp) {
    lastTstamp = tstamp;
  }

  void ProjectGPSForward(double tstamp, double curtime, double *gps) {
    double min_delta_tstamp = tstamp;
    PoseDatum *argmin_delta_tstamp = NULL;
    double min_delta_curtime = curtime;
    PoseDatum *argmin_delta_curtime = NULL;

    for (vector<PoseDatum*>::iterator it = posedata.begin();
	 it < posedata.end();
	 it++) {
      PoseDatum *pdatum = *it;
      double delta_tstamp = fabs(pdatum->timestamp - tstamp);
      if (delta_tstamp < min_delta_tstamp) {
	argmin_delta_tstamp = pdatum;
	min_delta_tstamp = delta_tstamp;
      }
	
      double delta_curtime = fabs(pdatum->timestamp - curtime);
      if (delta_curtime < min_delta_curtime) {
	argmin_delta_curtime = pdatum;
	min_delta_curtime = delta_curtime;
      }
    }

    if (argmin_delta_curtime != NULL && 
	argmin_delta_tstamp != NULL) {
      for (int i = 0; i < 6; i++) {
	gps[i] += argmin_delta_curtime->position[i] - 
	  argmin_delta_tstamp->position[i];
      }
    }

    for (vector<PoseDatum*>::iterator it = posedata.begin();
	 it < posedata.end();
	 it++) {
      delete *it;
    }

    posedata.clear();
  }
  
  double LastTimestamp() {
    return lastTstamp;
  }

 private:
  vector<PoseDatum*> posedata;
  double lastTstamp;
};

#endif
