/*
  lagr/OdDeviationLogger.h

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#ifndef __ODEVIATION_LOG_H__
#define __ODEVIATION_LOG_H__

#include <stdio.h>
#include <assert.h>
#include <gsl/gsl_cblas.h>
#include <string.h>

#include "../ModelMatrices.h"
#include "../Quaternion.h"

#define MAX_STATE_DIM 20

class OdDeviationLogger {
 public:
  OdDeviationLogger(char *fname) {
    log = fopen(fname, "w");
    assert(log != NULL);
  }

  void Update(double *new_state, double *reading, Quaternion *quat) {
    double expected_vel[3];
    vmat.ReplaceSubmatrixV(quat->GetMatrixF2B(), 3, 3);
    
    cblas_dgemv(CblasRowMajor, CblasNoTrans, 
		vmat.Rows(), vmat.Cols(), 1.0, 
		vmat.GetMatrix(), vmat.Cols(), 
		new_state, 1, 
		0.0, expected_vel, 1);


    for (int i = 0; i < vmat.Rows(); i++) {
      fprintf(log, "%.10lf ", reading[i] - expected_vel[i]);
    }
    fprintf(log, "\n");
  }

 private:
  FILE *log;
  VelMeasurementMatrix vmat;
};

#endif
