/*
  lagr/OfflinePoseFilter.h

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>

  Subclass of PoseRBFilter that accepts specially-formatted 
  messages from a log.
*/

#ifndef __OFFLINE_POSE_H__
#define __OFFLINE_POSE_H__

#include <string.h>

#include "../plotting/PlotMessage.h"
#include "../PoseRBFilter.h"
#include "../jittercorrector/JitterCorrector.h"
#include "GPSFixer.h"

#define DEFAULT_MESSAGE_SORTER_LAG 0.08

class LagrLogDatumWrapper {
 public:
  LagrLogDatumWrapper(char typea, double *dataptr, int count) {
    type = typea;
    memcpy(data, dataptr, sizeof(double)*count);
  }

  char type;
  double data[100];
};

class OfflinePoseFilter : public PoseRBFilter {
 public:
  OfflinePoseFilter(int particles, double lag = DEFAULT_MESSAGE_SORTER_LAG);

  void AddLogMessage(double timestamp, char type, double *data, int count);

  void AddSimLogMessage(double timestamp, char type, double *data, int count);

  void Randomize();

  void SetMeasurementUsage(measurement_t type, bool val);

 private:
  bool useAccel;
  bool useGyro;
  bool useGpsp;
  bool useGpsv;
  bool useOd;
  bool useMag;

  void SendPlotMessage(double *data, int count, plot_message_t type, int sockfd);

  JitterCorrector<LagrLogDatumWrapper*> msorter_log;
  GPSFixer gpsFixer;
};

#endif
