/*
  lagr/ProcessDeviationLogger.h

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#ifndef __PDEVIATION_LOG_H__
#define __PDEVIATION_LOG_H__

#include <stdio.h>
#include <assert.h>
#include <gsl/gsl_cblas.h>
#include <string.h>

#include "../ModelMatrices.h"

#define MAX_STATE_DIM 20

class ProcessDeviationLogger {
 public:
  ProcessDeviationLogger(char *fname, int state_dim) {
    log = fopen(fname, "w");
    assert(log != NULL);
    stateDim = state_dim;
  }

  void SetState(double *initial_state) {
    memcpy(state, initial_state, sizeof(double)*stateDim);
  }

  void Update(double *new_state) {
    double temp_state[MAX_STATE_DIM];

    cblas_dgemv(CblasRowMajor, CblasNoTrans, 
		pmat.Rows(), pmat.Cols(), 1.0, 
		pmat.GetMatrix(), pmat.Cols(), 
		state, 1, 
		0.0, temp_state, 1);

    for (int i = 0; i < stateDim; i++) {
      fprintf(log, "%.10lf ", new_state[i] - temp_state[i]);
    }
    fprintf(log, "\n");
    memcpy(state, new_state, sizeof(double)*MAX_STATE_DIM);
  }

 private:
  FILE *log;
  double state[MAX_STATE_DIM];
  int stateDim;
  ProcessMatrix pmat;
};

#endif
