/*
  lagr/pose_filter_process.cpp

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>

  Entry point process for pose filter that is directly run by the LAGR console.
  Expects four command line arguments: process slot that this process will 
  take, followed by three process slots specifying recipients of 
  pose messages.
*/

#include <arpa/inet.h>
#include <errno.h>
#include <gsl/gsl_cblas.h>
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#include <lagrInterface.h>

#include "LagrPoseFilter.h"
#include "../RBParticle.h"
#include "../plotting/PlotMessage.h"

//#define BIAS_ESTIMATION_TIMELIMIT (5.0)
//#define BIAS_MAGNITUDE_LIMIT 0.05
#define INITIAL_ANGLE_UNCERTAINTY 0.1

#define PARTICLES 100

static int doneG = 0;

#define BIND_ADDR "192.168.9.19" 
//#define SEND_ADDR "192.168.19.255"
#define DISPLAY_ADDR "192.168.9.255" // display messages sent here
//#define FORWARD_ADDR "192.168.19.255" // pose messages forwarded here
#define DISPLAY_PORT 19111
//#define FORWARD_PORT 19112

void init_api(int poseslot);

int main(int argc, char **argv) 
{
  LagrStatusType status;
  unsigned char *dataP;
  unsigned int size;
  LagrPoseFilter rbfilter_global(PARTICLES, 0.2);

  if (argc != 5) {
    printf("usage: pose_filter_process <this process's slot> <destination slots 1 - 3>\n");
    exit(1);
  }

  int poseslot = atoi(argv[1]);
  int plannerslot = atoi(argv[2]);
  int eyeslot1 = atoi(argv[3]);
  int eyeslot2 = atoi(argv[4]);

  rbfilter_global.AddDestSlot(plannerslot);
  rbfilter_global.AddDestSlot(eyeslot1);
  rbfilter_global.AddDestSlot(eyeslot2);
  rbfilter_global.SetDisplayHost(DISPLAY_ADDR, DISPLAY_PORT);
  //rbfilter_global.SetForwardHost(FORWARD_ADDR, FORWARD_PORT);

  init_api(poseslot);

  lagrCommSubscribe(lagrSubscriptionPosition);
  lagrCommSubscribe(lagrSubscriptionGPS);
  lagrCommSubscribe(lagrSubscriptionIMU);
  lagrCommSubscribe(lagrSubscriptionEncoder);

  while (!doneG) {
    LagrMessageType type;
    status = lagrCommReceive(&type, &dataP, &size);
    if (status != LAGR_OK) {
      printf("lagrCommReceive() failed: %d\n", status);
      break; 
    }

    rbfilter_global.AddLagrMessage(type, dataP);

    free(dataP);
  }

  // cleanup API
  lagrAPIDestroy();
}
 
void init_api(int poseslot) {
  LagrStatusType  status;

  status = lagrAPIInit((LagrProcessType)poseslot);
    
  // See If we were able to init the API
  if (status != LAGR_OK) {
    printf("lagrAPIInit() failed: %d\n", status);
    exit(1);
  }

  // watch termination signals in 'doneG'
  lagrWatchSignals(&doneG);
}
