/*
  lagr/pose_reader.cpp

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include <lagrInterface.h>

#include "lagr_penn.h"

static int doneG = 0;

void init_api(int pslot);

int main(int argc, char **argv) 
{
  if (argc != 2) {
    printf("usage: pose_filter_process <this process's slot>\n");
    exit(1);
  }

  int pslot = atoi(argv[1]);

  init_api(pslot);

  lagrCommSubscribe(lagrSubscriptionPosition);

  // loop until signal caught
  while (!doneG) {
    LagrMessageType type;
    
    unsigned int size;
    unsigned char *dataP;
    LagrStatusType status = lagrCommReceive(&type, &dataP, &size);
    if (status != LAGR_OK) {
      printf("lagrCommReceive() failed: %d\n", status);
      break; 
    }

    if (type == lagrMessageGeneric) {
      struct lagr_penn_header *header = (struct lagr_penn_header *)dataP;
      assert(header->magicNumber == 0x50454e30);
      /*
      LagrPoseState *pose = 
	(LagrPoseState*)(dataP + sizeof(struct lagr_penn_header));
      printf("LP %lf %lf %lf\n", 
	     (pose->localPose)[0],(pose->localPose)[1],(pose->localPose)[2]);
      printf("GP %lf %lf %lf\n", 
	     (pose->globalPose)[0],(pose->globalPose)[1],(pose->globalPose)[2]);
      */
    }

    free(dataP);
  }

  // cleanup API
  lagrAPIDestroy();
}
 
void init_api(int pslot) {
  LagrStatusType  status = lagrAPIInit((LagrProcessType)pslot);
    
  // See If we were able to init the API
  if (status != LAGR_OK) {
    printf("lagrAPIInit() failed: %d\n", status);
    exit(1);
  }

  // watch termination signals in 'doneG'
  lagrWatchSignals(&doneG);
}


