/*
  lagr/pose_test_log.cpp

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#include <glut.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>

#include "../UTM/LatLong-UTMconversion.h"
#include "../Quaternion.h"
#include "OfflinePoseFilter.h"
#include "../plotting/Plotter.h"
#include "../plotting/Menu.h"
#include "../plotting/IMUGraphic.h"
#include "../plotting/GPSPGraphic.h"
#include "../plotting/VelGraphic.h"

#define MESSAGE_LAG 0.4
//#define MESSAGE_LAG 0.0
#define PARTICLE_COUNT 100

#define STATE_DIM 6

bool profiling = false;
//bool interactive_plot = true;
int particle_display_count = 100;

FILE *logfile;
OfflinePoseFilter *rbfilter;
Plotter *plotter;

bool firstit = true;
bool lagr_posevalid = false, mine_posevalid = false;
double last_gpsv[3]; 
Quaternion *Lagr_Quat = NULL;
bool lagrHasFocus = false;

bool UseGPSp = true, UseGPSv = true, UseOd = true, UseGyro = true, UseAccel = false;

IMUGraphic *poseGraphic, *baselinePoseGraphic, *ukfPoseGraphic;
GPSPGraphic *gpsPosGraphic;
IMUGraphic *particleGraphics[PARTICLE_COUNT];

#define MAX(a,b) (a > b ? a : b)
#define MIN(a,b) (a < b ? a : b)

void process_log();
void init_rbfilter();
void reset_rbfilter();
void message_handler(user_message_t umess, char *messdata);

void init_rbfilter() {
  rbfilter = new OfflinePoseFilter(PARTICLE_COUNT, MESSAGE_LAG);
  rbfilter->Randomize();
}

void subtract_bias(double *number) {
  number[0] = int(floor(number[0])) % 10000 + number[0] - floor(number[0]);
  number[1] = int(floor(number[1])) % 10000 + number[1] - floor(number[1]);
}

void reset_rbfilter() {
  rewind(logfile);
  
  lagr_posevalid = mine_posevalid = false;
  
  delete rbfilter;
  init_rbfilter();
}

int main(int argc, char *argv[]) {
  init_rbfilter();

  if (argc < 2) {
    printf("usage: pose_test_log <input log file>\n");
    exit(1);
  }
  logfile = fopen(argv[1], "r");
    //  logfile = stdin;

  Lagr_Quat = new Quaternion(0.0, 0.0, 0.0);

  plotter = new Plotter(argc, argv, process_log, 800, 600);
  plotter->RegisterMenuMessageHandler(message_handler);

  poseGraphic = new IMUGraphic("pose", 2.5f, 1.0, 1.0, 0.0, 1.0);
  baselinePoseGraphic = 
    new IMUGraphic("bas_pose", 2.5f, 1.0, 0.5, 0.0, 1.0);
<<<<<<< .mine
  ukfPoseGraphic = new IMUGraphic("ukf", 2.5f, 0.5, 1.0, 0.5, 1.0);
=======
  //  ukfPoseGraphic = new IMUGraphic("ukf", 2.5f, 0.5, 1.0, 0.5, 1.0);
>>>>>>> .r139
  
  gpsPosGraphic = new GPSPGraphic();

  plotter->AddDrawable(poseGraphic);
  plotter->AddDrawable(baselinePoseGraphic);
  plotter->AddDrawable(gpsPosGraphic);
<<<<<<< .mine
  plotter->AddDrawable(ukfPoseGraphic);
=======
  //  plotter->AddDrawable(ukfPoseGraphic);
>>>>>>> .r139

  //  ukfPoseGraphic->SetTrailLength(20000);
  ukfPoseGraphic->SetTrailLength(20000);

  for (int i = 0; i < particle_display_count; i++) {
    char name[255];
    sprintf(name, "%d", i);
    particleGraphics[i] = new IMUGraphic(name, 0.5f, 1.0, 1.0, 1.0, 1.0);
    particleGraphics[i]->SetTrailLength(1);
    plotter->AddDrawable(particleGraphics[i]);
  }

  plotter->SetCamFocus(poseGraphic->GetPos());

  if (profiling == false)
    plotter->Start();
  else 
    while(1) process_log();
}

void message_handler(user_message_t umess, char *messdata) {

  switch (umess) {
  case (UMESS_ARB): {
    if (strcmp("switch", messdata) == 0) {
      if (lagrHasFocus) {
	printf("baseline\n");
	plotter->SetCamFocus(baselinePoseGraphic->GetPos());
      } else {
	plotter->SetCamFocus(poseGraphic->GetPos());
      }
      lagrHasFocus = !lagrHasFocus;
<<<<<<< .mine
    } else if (strcmp("ukf", messdata) == 0) {
      plotter->SetCamFocus(ukfPoseGraphic->GetPos());
=======
    } else if (strcmp("ukf", messdata) == 0) {
      //      plotter->SetCamFocus(ukfPoseGraphic->GetPos());
>>>>>>> .r139
    } else if (strcmp("gpsp", messdata) == 0) {
      UseGPSp = !UseGPSp;
      rbfilter->SetMeasurementUsage(R_GPSPOS, UseGPSp);
      printf("gps pos: %d\n", UseGPSp);
    } else if (strcmp("gpsv", messdata) == 0) {
      UseGPSv = !UseGPSv;
      rbfilter->SetMeasurementUsage(R_GPSVEL, UseGPSv);
      printf("gps vel: %d\n", UseGPSv);
    } else if (strcmp("od", messdata) == 0) {
      UseOd = !UseOd;
      rbfilter->SetMeasurementUsage(R_OD, UseOd);
      printf("od: %d\n", UseOd);
    } else if (strcmp("gyro", messdata) == 0) {
      UseGyro = !UseGyro;
      rbfilter->SetMeasurementUsage(R_GYRO, UseGyro);
      printf("gyro: %d\n", UseGyro);
    } else if (strcmp("accel", messdata) == 0) {
      UseAccel = !UseAccel;
      rbfilter->SetMeasurementUsage(R_GRAVONLY, UseAccel);
      printf("accel: %d\n", UseAccel);
    } else if (strstr(messdata, "chase")) {
      double chasedist[3];
      sscanf(messdata, "chase %lf %lf %lf", chasedist, chasedist + 1, chasedist + 2);
      plotter->SetCamChaseDist(chasedist);
    } else {
      printf("Unhandled user message '%s'", messdata);
    }
  }
    break;
  case (UMESS_RESET):
    reset_rbfilter();
    break;
  default:
    printf("unhandled message, id = %d\n", umess);
    break;
  }
}

void process_log() {
  bool draw_particles = true;
  //bool draw_particles = false;

  double readings[100], timestamp;
  static int counter = 0;
  char type;

  int rcount = fscanf(logfile, "%lf %c ", &timestamp, &type);

  if (feof(logfile) || rcount != 2) {
    if (profiling == true)
      exit(0);

    plotter->Pause();
    return;
  }

  static unsigned char pixbuf[800 * 600 * 3];

  //    if (delta_t > 0.0 ) 
  //    usleep(int(0.05*1e6));

  switch (type) {
  case 'Y': {
    rcount = fscanf(logfile, "%*f %*f %*f %lf %lf %lf\n", readings, readings + 1, readings + 2);
    assert(rcount == 3);
    double roll = readings[0], pitch = readings[1], yaw = readings[2];
    if (Lagr_Quat != NULL)
      delete Lagr_Quat;
    Lagr_Quat = new Quaternion(roll,pitch,yaw);

    baselinePoseGraphic->SetQuat(Lagr_Quat->GetArray());
  }
    break;
  case 'M': {
    rcount = fscanf(logfile, "%lf %lf %lf\n", readings, readings + 1, readings + 2);
    assert(rcount == 3);

    rbfilter->AddLogMessage(timestamp, type, readings, rcount);
  }
    break;
  case 'P': {
    lagr_posevalid = true;
    rcount = fscanf(logfile, "%*f %*f %*f %lf %lf %lf\n", readings, readings + 1, readings + 2);
    assert(rcount == 3);

    // HACK: make my pose and baseline pose appear on same level
    readings[2] = 0;


    subtract_bias(readings);


    //    baselinePoseGraphic->SetPos(readings);
  }
    break;
  case 'G': {
    rcount = fscanf(logfile, "%lf %lf %lf\n", readings, readings + 1, readings + 2);
    assert(rcount == 3);

    plotter->SetTime(timestamp);

    rbfilter->AddLogMessage(timestamp, type, readings, rcount);

    double pos[3];
    if (draw_particles) {
      rbfilter->NormalizeParticleWeights();
      double maxweight = rbfilter->MaxWeight();

      RBParticle **particles = rbfilter->Particles();
      for (int i = 0; i < particle_display_count; i++) {
	particleGraphics[i]->SetQuat(particles[i]->GetQuaternion()->GetArray());
	double alpha = particles[i]->GetWeight() / maxweight;
	particleGraphics[i]->SetColor(1.0, 1.0, 1 - alpha, 1.0);

	memcpy(pos, particles[i]->GetPosMean(), sizeof(double)*3);
	//	pos[2] *= -1;



	subtract_bias(pos);

	//			pos[2] = 0;
	particleGraphics[i]->SetPos(pos);


	//		double *covar = particles[i]->GetPosCovar();
	//		double maxcov = max(covar[0], covar[6*1 + 1]);
	//	particleGraphics[i]->SetCovar(maxcov);

	//	printf("C %lf %lf\n", covar[0], covar[6*1 + 1]);

      }
    }
  }
    break;
  case 'A': {
    rcount = fscanf(logfile, "%lf %lf %lf\n", readings, readings + 1, readings + 2);
    assert(rcount == 3);

    rbfilter->AddLogMessage(timestamp, type, readings, rcount);
  }
    break;
  case 'V': {

    rcount = fscanf(logfile, "%lf %lf %lf", readings, readings + 1, readings + 2);
    assert(rcount == 3);

    poseGraphic->GetVelGraphic()->SetVel(readings);

    rbfilter->AddLogMessage(timestamp, type, readings, rcount);

    rbfilter->Resample();

    
    RBParticle *estimate = rbfilter->Estimate();

    poseGraphic->SetQuat(estimate->GetQuaternion()->GetArray());

    double pos[3];
    memcpy(pos, estimate->GetPosMean(), sizeof(double)*3);

    //    fprintf(stderr, "%lf %lf %lf\n", pos[0], pos[1], pos[2]);

<<<<<<< .mine
    //    printf("E %lf %lf %lf\n", pos[0], pos[1], pos[2]);
=======
    ///    printf("E %lf %lf %lf\n", pos[0], pos[1], pos[2]);
>>>>>>> .r139

<<<<<<< .mine
            pos[2] = 0;
=======
    //    pos[2] = 0;
>>>>>>> .r139


    subtract_bias(pos);

    poseGraphic->SetPos(pos);



    // write images to file
    //    if ((counter++ % 4) == 0) {
    if (0) {
      char namebuf[255];
      glReadPixels(0, 0, 640, 480, GL_RGB, GL_UNSIGNED_BYTE, pixbuf);
      sprintf(namebuf, "%d.ppm", counter);
      FILE *file = fopen(namebuf, "w");
      fprintf(file, "P6\n640 480\n255\n");
      for (int i = 479; i >= 0; i--) {
	fwrite(pixbuf + 640 * 3 *i, 1, 640 * 3, file);
      }
      fclose(file);
    }

  }
    break;
  case 'S': {
    rcount = fscanf(logfile, "%lf %lf %lf %lf %lf %lf %lf %lf\n", 
		    readings, readings + 1, readings + 2,
		    readings + 3, readings + 4, readings + 5,
		    readings + 6, readings + 7);
    assert(rcount == 8);

    memcpy(last_gpsv, readings + 5, sizeof(double)*3);





    double pos_xyz_covar[5];
    LLtoUTMrad(readings[0], readings[1], pos_xyz_covar[0], pos_xyz_covar[1]);
    pos_xyz_covar[2] = readings[2]; pos_xyz_covar[3] = readings[3];
    pos_xyz_covar[4] = readings[4];

    //    pos_xyz_covar[2] = 0;

<<<<<<< .mine
    //    printf("G %lf %lf %lf\n", pos_xyz_covar[0], pos_xyz_covar[1], pos_xyz_covar[2]);

    pos_xyz_covar[2] = 0;

=======
>>>>>>> .r139

    subtract_bias(pos_xyz_covar);



    gpsPosGraphic->SetPosErr(pos_xyz_covar, pos_xyz_covar + 3);

    poseGraphic->GetGPSVGraphic()->SetVel(readings + 5);
    baselinePoseGraphic->GetGPSVGraphic()->SetVel(readings + 5);

    rbfilter->AddLogMessage(timestamp, type, readings, rcount);

  }
    break;
  case 'Z': {
    rcount = fscanf(logfile, "%lf %lf %lf %lf %lf %lf %lf %lf\n", 
		    readings, readings + 1, readings + 2,
		    readings + 3, readings + 4, readings + 5,
		    readings + 6, readings + 7);
    assert(rcount == 8);

    memcpy(last_gpsv, readings + 5, sizeof(double)*3);

    subtract_bias(readings);

    gpsPosGraphic->SetPosErr(readings, readings + 3);

    poseGraphic->GetGPSVGraphic()->SetVel(readings + 5);

    //    baselinePoseGraphic->GetGPSVGraphic()->SetVel(readings + 5);

    rbfilter->AddLogMessage(timestamp, type, readings, rcount);

  }
    break;
<<<<<<< .mine
  case 'T': {
    rcount = fscanf(logfile, "%lf %lf %lf %lf %lf %lf\n", readings, readings + 1, readings + 2, readings + 3, readings + 4, readings + 5);
    Quaternion quat(readings[0], readings[1], readings[2]);

    subtract_bias(readings + 3);
   
    //    readings[5] = 0.0;

    baselinePoseGraphic->SetPos(readings + 3);
    baselinePoseGraphic->SetQuat(quat.GetArray());


    RBParticle *estimate = rbfilter->Estimate();
    //    double *estpos = estimate->GetPosMean();
    double esteulers[3];
    estimate->GetQuaternion()->GetEulerAngles(esteulers);

    //    double angdiff = estimate->GetQuaternion()->GetAngleDiff(&quat);

    if ((counter++ % 5) == 0) {
      /*
      printf("%lf %lf %lf %lf ",
	     angdiff, estpos[0], estpos[1], estpos[2]);
      printf("%lf %lf %lf\n",
	     readings[3], readings[4], readings[5]);
      */
    }

    break;
  }
  case 'K': {
    rcount = fscanf(logfile, "%lf %lf %lf %lf %lf %lf\n", readings, readings + 1, readings + 2, readings + 3, readings + 4, readings + 5);
    Quaternion quat(readings[0], readings[1], readings[2]);

    //    printf("ukf %lf %lf %lf\n", readings[3], readings[4], readings[5]);

    subtract_bias(readings + 3);
   
    //    readings[5] = 0.0;

    ukfPoseGraphic->SetPos(readings + 3);
    ukfPoseGraphic->SetQuat(quat.GetArray());
    break;
  }
=======
  case 'T': {
    rcount = fscanf(logfile, "%lf %lf %lf %lf %lf %lf\n", readings, readings + 1, readings + 2, readings + 3, readings + 4, readings + 5);
    Quaternion quat(readings[0], readings[1], readings[2]);

    subtract_bias(readings + 3);
   
    //    readings[5] = 0.0;

    baselinePoseGraphic->SetPos(readings + 3);
    baselinePoseGraphic->SetQuat(quat.GetArray());


    RBParticle *estimate = rbfilter->Estimate();
    //    double *estpos = estimate->GetPosMean();
    double esteulers[3];
    estimate->GetQuaternion()->GetEulerAngles(esteulers);

    //    double angdiff = estimate->GetQuaternion()->GetAngleDiff(&quat);

    /*
    if ((counter++ % 5) == 0) {
      printf("%lf %lf %lf %lf ",
	     angdiff, estpos[0], estpos[1], estpos[2]);
      printf("%lf %lf %lf\n",
	     readings[3], readings[4], readings[5]);
    }
    */

    break;
  }
  case 'K': {
    rcount = fscanf(logfile, "%lf %lf %lf %lf %lf %lf\n", readings, readings + 1, readings + 2, readings + 3, readings + 4, readings + 5);
    Quaternion quat(readings[0], readings[1], readings[2]);

    printf("ukf %lf %lf %lf\n", readings[3], readings[4], readings[5]);

    subtract_bias(readings + 3);
   
    //    readings[5] = 0.0;

    //    ukfPoseGraphic->SetPos(readings + 3);
    //    ukfPoseGraphic->SetQuat(quat.GetArray());
    break;
  }
>>>>>>> .r139
  default:
    printf("you MUST handle all cases (%c)\n", type);
    exit(1);
    break;
  }

  plotter->Update();

}
