/*
  lagr/simul_plot.cpp

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>

#include "../Quaternion.h"

#include "../IMURBFilter.h"

#define PARTICLE_COUNT 100

int main(int argc, char *argv[]) {
  IMURBFilter rbfilter(PARTICLE_COUNT, false);

  double readings[3], timestamp;
  char type;

  /*
  if (argc < 2) {
    printf("usage: plot_lagr_log <log file>\n");
    exit(1);
  }
    FILE *logfile = fopen(argv[1], "r");
  */
  FILE *logfile = stdin;

  double lagrpos[3], lagrquat[4];
  double last_t = 0.0;
  double initangles[3];
  int counter = 0;
  bool firstangles = true;
  while (fscanf(logfile, "%lf %c %lf %lf %lf\n", 
		&timestamp, &type, readings, readings + 1, readings + 2) == 5) {

    double delta_t = 0.0;
    if (last_t != 0.0) 
      delta_t = timestamp - last_t; 
    last_t = timestamp;

    //    if (delta_t > 0.0 ) 
    //    usleep(int(0.05*1e6));

    switch (type) {
    case 'Y': {
      double roll = readings[0], pitch = readings[1], yaw = readings[2];
      //      double roll = 0.0, pitch = 0.0, yaw  = readings[2];
      if (firstangles == true) {
	for (int i = 0; i < 3; i++) {
	  initangles[i] = readings[i];
	}
	firstangles = false;
      }
      Quaternion quat(roll - initangles[0], 
		      pitch - initangles[1], 
		      yaw - initangles[2]);
      //      quat.Invert();
      memcpy(lagrquat, quat.GetArray(), sizeof(double)*4);
    }
      break;
    case 'P': {
      //      usleep(10000);
      memcpy(lagrpos, readings, sizeof(double)*3);
    }
      break;
    case 'G':
      rbfilter.Update(readings, R_GYRO, timestamp);
      break;
    case 'A':
      //      rbfilter.Update(readings, R_ACCEL, timestamp);
      rbfilter.Update(readings, R_GRAVONLY, timestamp);
      break;
    case 'V':
      /*
      readings[0] *= -1;
      readings[1] *= -1;
      readings[2] *= -1;
      rbfilter.Update(readings, R_GPSVEL, timestamp);
      */
      //      readings[0] = 2.0;
      rbfilter.Update(readings, R_VEL, timestamp);
      break;
    }

    if (counter++ % 50 == 0) {
      rbfilter.Resample();
    }


    printf("2\n");
    RBParticle *estimate = rbfilter.Estimate();
    double *est_quat = estimate->GetQuaternion()->GetArray();
    double *est_pos = estimate->GetPosMean();
    printf("%lf %lf %lf %lf %lf %lf %lf\n", 
	   est_quat[0], est_quat[1], est_quat[2], est_quat[3],
	   est_pos[0], est_pos[1], est_pos[2]);
    
    printf("%lf %lf %lf %lf %lf %lf %lf\n", 
	   lagrquat[0], lagrquat[1], lagrquat[2], lagrquat[3],
	   lagrpos[0], lagrpos[1], lagrpos[2]);

    
    if (counter % 30 == 0)
      fprintf(stderr, "P-Z M %lf X %lf\n", est_pos[2], lagrpos[2]);
    fflush(stdout);
  }
}
