/*
  lagr/test_messages.cpp

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include <lagrInterface.h>

#define PROCESS_SLOT 3

static int doneG = 0;

void init_api(void);

int main(int argc, char **argv) 
{
  LagrStatusType status;
  unsigned char *dataP;
  unsigned int size;

  init_api();

  // subscribe to some data types
  lagrCommSubscribe(lagrSubscriptionPosition);
  lagrCommSubscribe(lagrSubscriptionGPS);
  lagrCommSubscribe(lagrSubscriptionIMU);

  fcntl(1, F_SETFL, O_NONBLOCK);

  // loop until signal caught
  while (!doneG) {
    LagrMessageType type;
    LagrPoseState *poseP;
    //	LagrRCState   *rcP;
    LagrGPS *gpsP;
    LagrIMUMessage *imu_message; 

    if (getchar() != EOF) {
      printf("Exiting\n");
      break;
    }

    // block until message arrives
    status = lagrCommReceive(&type, &dataP, &size);
    if (status != LAGR_OK) {
      printf("lagrCommReceive() failed: %d\n", status);
      break; 
    }

    double timestamp = 0.0;

    switch (type) {
    case lagrMessageIMU:
      imu_message = (LagrIMUMessage*)dataP;

      for (int i = 0; i < imu_message->count; i++) {
	LagrIMU *imu_state = &(imu_message->samplesP[i]);
	timestamp = imu_state->timeStamp;
	printf("%lf IMU %lf %lf %lf\n", timestamp,
	       imu_state->xAcceleration, imu_state->yAcceleration, imu_state->zAcceleration);
	break;
      }
      break;
    case lagrMessagePosition: 
      poseP = (LagrPoseState*)dataP;
      timestamp = poseP->timeStamp; 
      printf("%lf Pose\n", timestamp);
      break;
    case lagrMessageGPS:
      gpsP = (LagrGPS*)dataP;
      timestamp = gpsP->timeStamp;
      printf("%lf GPS\n", timestamp);
      break;
    default:
      break;
    }
  }

  // cleanup API
  lagrAPIDestroy();
}
 
void init_api() {
  LagrStatusType  status;

  status = lagrAPIInit((LagrProcessType)PROCESS_SLOT);
    
  // see if we were able to init the API
  if (status != LAGR_OK) {
    printf("lagrAPIInit() failed: %d\n", status);
    exit(1);
  }

  // watch termination signals in 'doneG'
  lagrWatchSignals(&doneG);
}


