/*
  plotting/CamFilter.cpp

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#include "CamFilter.h"

#include <math.h>
#include <string.h>

CamFilter::CamFilter(double param) {
  memory[0] = memory[1] = memory[2] = 0.0;
  fparam = param;
}

double *CamFilter::Update(double *readings) {
  double delta[3];
  
  for (int i = 0; i < 3; i++) 
    delta[i] = readings[i] - memory[i];

  double distance = 
    sqrt(delta[0]*delta[0] + delta[1]*delta[1] + delta[2]*delta[2]);

  // fast catch-up if view is too far away
  if (distance < 100) {
    for (int i = 0; i < 3; i++) {
      memory[i] = fparam * readings[i] + (1 - fparam) * memory[i];
    }
  } else {
    for (int i = 0; i < 3; i++) 
      memory[i] = readings[i];
  }
  return memory;
}

void CamFilter::SetPos(double *in) {
  memcpy(memory, in, sizeof(double)*3);
}
