/*
  plotting/IMUGraphic.cpp

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#include <glut.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "../Quaternion.h"

#include "IMUGraphic.h"

IMUGraphic::IMUGraphic(const char *namestr, float lw, double r, double g, double b, double a) :
  plotCovar(false)
{

  strncpy(name, namestr, MAX_IG_NAME_SIZE);

  maxTrailLength = DEFAULT_MAX_TRAIL_LEN;

  listId = glGenLists(1);
  glNewList(listId, GL_COMPILE);

  glPushMatrix();

  double z_lower = -0.25;
  double z_upper = -z_lower;
  double x_width = 1.0;
  double y_width = 0.7;
  glBegin(GL_LINE_STRIP);
  glVertex3d(-x_width/2, -y_width/2, z_lower);
  glVertex3d(-x_width/2, y_width/2, z_lower);
  glVertex3d(x_width/2, y_width/2, z_lower);
  glVertex3d(x_width/2, -y_width/2, z_lower);
  glVertex3d(-x_width/2, -y_width/2, z_lower);
  glEnd();
  glBegin(GL_LINE_STRIP);
  glVertex3d(-x_width/2, -y_width/2, z_upper);
  glVertex3d(-x_width/2, y_width/2, z_upper);
  glVertex3d(x_width/2, y_width/2, z_upper);
  glVertex3d(x_width/2, -y_width/2, z_upper);
  glVertex3d(-x_width/2, -y_width/2, z_upper);
  glEnd();
  glBegin(GL_LINES);
  glVertex3d(-x_width/2, -y_width/2, z_lower);  
  glVertex3d(-x_width/2, -y_width/2, z_upper);
  glVertex3d(x_width/2, -y_width/2, z_lower);  
  glVertex3d(x_width/2, -y_width/2, z_upper);
  glVertex3d(-x_width/2, y_width/2, z_lower);  
  glVertex3d(-x_width/2, y_width/2, z_upper);
  glVertex3d(x_width/2, y_width/2, z_lower);  
  glVertex3d(x_width/2, y_width/2, z_upper);
  glEnd();
  
  glBegin(GL_LINES);
  glVertex3d(0.0, 0.0, 0.0);  
  glVertex3d(x_width/2, 0.0, 0.0);
  glVertex3d(0.0, 0.0, 0.0);  
  glVertex3d(0.0, y_width/2, 0.0);
  glVertex3d(0.0, 0.0, 0.0);  
  glVertex3d(0.0, 0.0, z_upper);
  glEnd();
  
  glRasterPos3d(x_width/2, 0.0, 0.0);
  glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, '+');
  glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, 'x');
  glRasterPos3d(0.0, y_width/2, 0.0);
  glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, '+');
  glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, 'y');
  glRasterPos3d(0.0, 0.0, z_upper);
  glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, '+');
  glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, 'z');
  glPopMatrix();
  
  glEndList();

  SetColor(r,g,b,a);
  lineWidth = lw;

  gpsVGraphic = new VelGraphic();
  velGraphic = new VelGraphic();

  double zeroes[3] = { 0.0, 0.0, 0.0 };
  velGraphic->SetVel(zeroes);
  gpsVGraphic->SetVel(zeroes);
}

void IMUGraphic::Draw() {
  glPushMatrix();

  glLineWidth(lineWidth * 2.0);

  glColor4d(red, green, blue, alpha);

  glBegin(GL_LINE_STRIP);
  vector<IMUPathPoint*>::iterator path_it = imuPath.begin();
  for (++path_it; path_it < imuPath.end(); path_it++) {
    glVertex3dv((*path_it)->point);
  }
  glEnd();

  glColor3d(0.0, 1.0, 1.0);

  gpsVGraphic->SetPos(position);
  gpsVGraphic->Draw();

  glTranslated(position[0], position[1], position[2]);

  if (plotCovar) {
    glColor4d(red, green, blue, alpha / 10.0);
    //    glutSolidSphere(covar, 15, 5);
    glBegin(GL_LINE_STRIP);
    for (double pangle = 0.0; pangle <= 2 * M_PI; pangle += 2 * M_PI / 10)
      glVertex3d(covar * cos(pangle), covar * sin(pangle), 0);
    glEnd();
  }

  glColor4d(red, green, blue, alpha);

  Quaternion quat(quaternion[0], quaternion[1], quaternion[2], quaternion[3]);
  /*
  double eulers[3]; 
  quat.GetEulerAngles(eulers);
  glRotated(180. * eulers[0] / M_PI, 1.0, 0.0, 0.0);
  glRotated(180. * eulers[1] / M_PI, 0.0, 1.0, 0.0);
  glRotated(180. * eulers[2] / M_PI, 0.0, 0.0, 1.0);
  */

  glRotated(180.0 * (2 * acos(quaternion[3])) / M_PI, 
	    quaternion[0],
	    quaternion[1],
	    quaternion[2]);


  glRotated(0.0, 0.0, 0.0, 1.0);
  glCallList(listId);
  glColor3d(1.0, 1.0, 1.0);

  glRasterPos3d(0.0, 0.0, 1.0);
  velGraphic->Draw();

  /*
  char posbuf[255];
  sprintf(posbuf, "[%5f, %5g, %5g] %s", position[0], position[1], position[2], name);
  //  sprintf(posbuf, "[%s]", name);
  char *pstring = posbuf;
  char dispchar = *pstring++;
  while (dispchar != 0) {
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, dispchar);
    dispchar = *pstring++;
  }
  */

  velGraphic->Draw();

  glPopMatrix();
}
