/*
  plotting/IMUGraphic.h

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#ifndef __IMU_GRAPHIC_H__
 #define __IMU_GRAPHIC_H__
 
 #include <glut.h>
 #include <string.h>
 #include <vector>
 using namespace std;
 
 #include "GLDrawable.h"
 #include "VelGraphic.h"
 
 #define MAX_IG_NAME_SIZE 255
 
 #define DEFAULT_MAX_TRAIL_LEN 100
 
 class IMUPathPoint {
  public:
   IMUPathPoint(double *pos) {
     memcpy(point, pos, sizeof(double)*3);
   }
 
   double point[3];
 };
 
 
 class IMUGraphic : public GLDrawable {
  public:
   IMUGraphic(const char *name, float lw, double r, double g, double b, double a);
 
   virtual ~IMUGraphic() {
     ErasePath();
     delete gpsVGraphic;
     delete velGraphic;
   }
 
   VelGraphic *GetGPSVGraphic() { return gpsVGraphic; }
   VelGraphic *GetVelGraphic() { return velGraphic; }
 
   void ErasePath() {
     for (vector<IMUPathPoint*>::iterator path_it = imuPath.begin();
 	 path_it < imuPath.end(); path_it++) {
       delete (*path_it);
     }
     imuPath.clear();
   }
 
   void SetColor(double r, double g, double b, double a) {
     red = r; green = g; blue = b; alpha = a;
   }
 
   void SetCovar(double cov) {
     covar = sqrt(cov);
     plotCovar = true;
   }

   void SetQuat(double *quat) {
     memcpy(quaternion, quat, sizeof(double)*4);
   }
 
   void SetPos(double *pos) {
     memcpy(position, pos, sizeof(double)*3);
 
     while (imuPath.size() >= maxTrailLength) {
       IMUPathPoint *point = imuPath[0];
       delete point;
       imuPath.erase(imuPath.begin());
     }
     imuPath.push_back(new IMUPathPoint(pos));
   }
 
   double *GetPos() { return position; }
 
   void Draw();
 
   void SetTrailLength(int tlen) { maxTrailLength = tlen; }
 
  private:
   GLuint listId;
   double quaternion[4];
   double position[3];
   double red, green, blue, alpha;
   float lineWidth;
   char name[MAX_IG_NAME_SIZE];
   VelGraphic *gpsVGraphic, *velGraphic;
   unsigned int maxTrailLength;
   bool plotCovar;
   double covar;
 
   vector<IMUPathPoint*> imuPath;
 };
 
 #endif
