/*
  plotting/LocalMapGraphic.h

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#ifndef __LMAP_GRAPHIC_H__
#define __LMAP_GRAPHIC_H__

#include <ext/hash_map>
using namespace ::__gnu_cxx;
#include <string.h>
#include <math.h>

#include "GLDrawable.h"

struct eqstr {
  bool operator()(const char *s1, const char *s2) {
    return (strcmp(s1, s2) == 0);
  }
};

typedef hash_map<const char *, double*, hash<const char *>, eqstr> stringhash_t;

class LocalMapGraphic : public GLDrawable {
 public:
  LocalMapGraphic(double cellsize) {
    cellSize = cellsize;
  };
  
  virtual ~LocalMapGraphic() {
    for (stringhash_t::iterator hash_it = obstacleMap.begin();
	 hash_it != obstacleMap.end();
	 hash_it++) {
      delete hash_it->first;
      delete hash_it->second;
    }
  }

  /*
  void SetData(double *mdata) {
    memcpy(map, mdata, sizeof(double)*MAPROWS*MAPCOLS);
  }
  */

  void SetCell(double *xyval, double zelev) {
    zHeight = zelev;

    char *hashkey = new char[255];
    //    char hashkey[255];
    snprintf(hashkey, 255, "%.2f %.2f", xyval[0], xyval[1]);

    stringhash_t::iterator hash_it = obstacleMap.find(hashkey);

    if (hash_it != obstacleMap.end()) {
      obstacleMap.erase(hash_it);
      delete hash_it->first;
      delete hash_it->second;
    }

    double *keydata = new double[3];
    memcpy(keydata, xyval, sizeof(double)*3);
    obstacleMap[hashkey] = keydata;
  }

  void Draw();

 private:
  stringhash_t obstacleMap;
  double zHeight;
  double cellSize;
};

#endif
