/*
  plotting/Menu.cpp

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#include <glut.h>
#include <string.h>
#include <stdio.h>

#include "Menu.h"

void Menu::Draw() {
  glColor4d(1.0, 1.0, 1.0, 0.9);

  char outbuf[MAX_USER_INPUT_BUFFER];

  if (userInputActive) {
    userDataEntry[userDataEntryPos] = 0;
    sprintf(outbuf, "%s: %s", menuMessageBuffer, userDataEntry);
    DrawString(outbuf, 0, 0);
  } else {
    int text_x_pos = 10;
    int text_y_pos = int(0.75 * winHeight);
    int text_y_delta = 20;
    
    switch (menuState) {
    case (MENU_OFF):
      break;
    case (MENU_MESSAGE):
      DrawString(menuMessageBuffer, 0, 0);
      break;
    case (MENU_MAIN):
      /*
      DrawString("Camera chase (d)istance", text_x_pos, text_y_pos);
      DrawString("Arbitrary (c)ommand", text_x_pos, text_y_pos + text_y_delta);
      */
      DrawString("North is towards top of screen", text_x_pos, text_y_pos);
      DrawString("Left click zooms in", text_x_pos, text_y_pos + text_y_delta);
      DrawString("Shift + left click zooms out", text_x_pos, text_y_pos + 2*text_y_delta);
      break;
    }
  }

  glMatrixMode(GL_MODELVIEW);
}

void Menu::DispMessage(const char *message) {
  sprintf(menuMessageBuffer, "%s", message);
  menuState = MENU_MAIN;
}

void Menu::DrawString(const char *pstring, int xpos, int ypos) {
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  glOrtho(0,winWidth,0,winHeight,-1.0,1.0);

  //  glDisable(GL_DEPTH_TEST);

  glRasterPos2i(xpos, ypos);

  char dispchar = *pstring++;
  while(dispchar != '\0') {
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, dispchar);
    dispchar = *pstring++;
  }

  glPopMatrix();
}

user_message_t Menu::HandleKeystroke(unsigned char key, int xpos, int ypos) {
  user_message_t umess = UMESS_NONE;

  if (userInputActive) {
    if (key == '\r') {
      userInputActive = false;
      userDataEntryPos = 0;
      switch (inputState) {
      case (INPUT_CAMPOS): {
	double *chase_dist = (double*)userMessage;
	sscanf(userDataEntry, "%lf %lf %lf", 
	       chase_dist, chase_dist + 1, chase_dist + 2);
	umess = UMESS_CAMPOS;
      }
	break;
      case (INPUT_ARBCOMMAND):
	umess = UMESS_ARB;
	strncpy(userMessage, userDataEntry, MAX_USER_INPUT_BUFFER);
	break;
      }
    } else {
      userDataEntry[userDataEntryPos++] = key;
      userDataEntry[userDataEntryPos] = 0;
      return umess;
    }
  }

  switch (menuState) {
  case (MENU_OFF):
  case (MENU_MAIN):
    switch (key) {
    case 's':
      umess = UMESS_SINGLESTEP;
      break;
    case 'p':
      umess = UMESS_PAUSE;
      break;
    case 'R':
      umess = UMESS_RESET;
      break;
    case 'r':
      umess = UMESS_RESUME;
      break;
    case 'q':
      umess = UMESS_EXIT;
      break;
    case 'c':
      inputState = INPUT_ARBCOMMAND;
      userInputActive = true;
      sprintf(menuMessageBuffer, "Command");
      break;
    default:
      menuState = MENU_MAIN;
      break;
    };
    break;
    /*
  case (MENU_MAIN): 
    switch (key) {
    case 'q':
      //      menuState = MENU_OFF;
      umess = UMESS_EXIT;
      break;
    case 'd':
      inputState = INPUT_CAMPOS;
      userInputActive = true;
      sprintf(menuMessageBuffer, "Chase distance (x y z)");
      break;
    case 'c':
      inputState = INPUT_ARBCOMMAND;
      userInputActive = true;
      sprintf(menuMessageBuffer, "Command");
      break;
    default:
      printf("NYI\n");
      break;
    }
    */
  default:
    break;
  }

  return umess;
}
