/*
  plotting/PlotMessage.h

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#ifndef __PLOT_MESSAGE_H__
#define __PLOT_MESSAGE_H__

#define MAX_MESSAGE_SIZE 300
#define MAX_PMESSAGE_AS_STRING 2048

#include <assert.h>
#include <stdio.h>
#include <string.h>

typedef enum { PLOT_GPS, PLOT_POSE, PLOT_BASELINEPOSE, PLOT_LOCALPOSE, 
	       PLOT_PATH, PLOT_LOCALMAP, PLOT_VEL,
	       PLOT_IGNORE, PLOT_UNKNOWN, 
	       PLOT_END_OF_LIST } plot_message_t;

typedef struct {
  const char *messageprefix;
  plot_message_t messtype;
} message_type_data_t;

class PlotMessage {
 public:
  PlotMessage() {
  };

  PlotMessage(plot_message_t typea, size_t size, void *data) {
    type = typea;
    memcpy(buf, data, size);
  }

  PlotMessage(char *stringified);

  plot_message_t Type() { return type; }
  
  char *Data() { return buf; }

  char *AsString() {
    double *dbuf = (double*)buf;

    switch (type) {
    case PLOT_GPS:
      snprintf(strbuf, MAX_PMESSAGE_AS_STRING, 
	       "GPS %lf %lf %lf %lf %lf %lf %lf %lf\n",
	       dbuf[0], dbuf[1], dbuf[2], dbuf[3], 
	       dbuf[4], dbuf[5], dbuf[6], dbuf[7]);
      break;
    case PLOT_VEL:
      snprintf(strbuf, MAX_PMESSAGE_AS_STRING, 
	       "VEL %lf %lf %lf\n", 
	       dbuf[0], dbuf[1], dbuf[2]);
      break;
    case PLOT_POSE:		/* xyz and Euler angles */
      snprintf(strbuf, MAX_PMESSAGE_AS_STRING, 
	      "CGPOSE %lf %lf %lf %lf %lf %lf\n",
	      dbuf[0], dbuf[1], dbuf[2], 
	      dbuf[3], dbuf[4], dbuf[5]);
      break;
    case PLOT_BASELINEPOSE:
      snprintf(strbuf, MAX_PMESSAGE_AS_STRING, 
	      "BGPOSE %lf %lf %lf %lf %lf %lf\n",
	       dbuf[0], dbuf[1], dbuf[2], 
	       dbuf[3], dbuf[4], dbuf[5]);
      break;
    case PLOT_LOCALPOSE:
      snprintf(strbuf, MAX_PMESSAGE_AS_STRING, 
	      "CLPOSE %lf %lf %lf %lf %lf %lf %lf\n",
	      dbuf[0], dbuf[1], dbuf[2], 
	      dbuf[3], dbuf[4], dbuf[5], dbuf[6]);
      break;
    default:
      // FIXME
      assert(0);
      break;
    }

    return strbuf;
  }

  int DataCount() { return dataCount; }

 private:
  plot_message_t type;
  char buf[MAX_MESSAGE_SIZE];
  char strbuf[MAX_PMESSAGE_AS_STRING];
  int dataCount;
};

#endif
