/*
  plotting/Plotter.h

  (C) 2005 Paul Vernaza, <vernaza@stwing.upenn.edu>
*/

#ifndef __PLOTTER_H__
#define __PLOTTER_H__

#include <assert.h>
#include <string.h>
/*
#include <vector>
using namespace std;
*/

#include "GLDrawable.h"
#include "GridGraphic.h"
#include "CamFilter.h"
#include "Menu.h"

#define MAX_DRAWABLES 2048

/*
typedef enum { PLOTTER_GRID, PLOTTER_PUBLIC_ID_BASE } 
plotter_private_drawable_ids_t;
*/

class Plotter {
 public:
  Plotter(int argc, char *argv[],
	  void (*idle_func)(void),
	  int width, int height, bool enable_aa = true);

  void Start();

  void Update();
  void Pause();
  void Resume();

  void SetSingleStepping(bool val) { singleStepping = val; }

  void RegisterMenuMessageHandler(void (*func)(user_message_t mess, 
					       char *mdata)) {
    menuHandler = func;
  }

    //  void RegisterResetCallback(void (*func)(void)) { resetFunc = func; }

  int AddDrawable(GLDrawable *drawable) {
    assert(drawableCount < MAX_DRAWABLES);
    drawables[drawableCount] = drawable;
    drawableCount++;
    drawable->SetEnabled(true);
    return drawableCount - 1;
  }

  GLDrawable* GetDrawable(int id) { return drawables[id]; }

  /*
  void SetDrawable(int id, GLDrawable *drawable) {
    assert(id + PLOTTER_PUBLIC_ID_BASE < MAX_DRAWABLES);
    drawables[id + PLOTTER_PUBLIC_ID_BASE] = drawable;
    drawableCount++;
  }

  GLDrawable* GetDrawable(int id) { 
    assert(id + PLOTTER_PUBLIC_ID_BASE < MAX_DRAWABLES);
    return drawables[id + PLOTTER_PUBLIC_ID_BASE]; 
  }
  */

  void SetCamChaseDist(double *chase) { 
    memcpy(camChaseDist, chase, sizeof(double)*3);
  };

  void SetCamFocus(double *cf) { camFocus = cf; }

  void SetTime(double timestamp) { currentTime = timestamp;  }

 private:
  void (*idleFunc)(void);
  void (*menuHandler)(user_message_t umess, char *mdata);
  CamFilter camFilter;
  GridGraphic *gridGraphic;
  Menu menu;
  int winWidth, winHeight;
  int stepCounter;
  int stepSize;
  GLDrawable *drawables[MAX_DRAWABLES];
  int drawableCount;

  bool singleStepping;
  bool lagrFocus;

  double camChaseDist[3];
  double *camFocus;
  double currentTime;

  static int magicNum;
  static Plotter* currentObject;
  
  void InitGL(int argc, char *argv[], bool enable_aa);
  void ResizeScene(int w, int h);
  void KeyboardHandler(unsigned char key, int xpos, int ypos);
  void MouseHandler(int button, int state, int x, int y);
  void DrawScene();
  void DrawMenu();
  void DrawTime();
  void DrawString(const char *pstring, int xpos, int ypos);

  static void wResizeScene(int w, int h) {
    Plotter::currentObject->ResizeScene(w, h);
  }
  static void wKeyboardHandler(unsigned char key, int xpos, int ypos) {
    Plotter::currentObject->KeyboardHandler(key, xpos, ypos);
  }
  static void wMouseHandler(int button, int state, int x, int y) {
    Plotter::currentObject->MouseHandler(button, state, x, y);
  }
  static void wDrawScene() {
    Plotter::currentObject->DrawScene();
  }
};

#endif
